/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkcommon/mbs.h"
#include "parameters.h"

using namespace Tk;
using namespace Experimenter;

Parameters:: Parameters(Db::Database * database)
  : Gtk::Frame("Parameters"),
    _horizon(&_speed)
{
  set_border_width(4);
  
  Gtk::HBox *box = Gtk::manage( new Gtk::HBox() );
  add(*box);
  box->set_spacing(4);
  
  Gtk::Table *table = Gtk::manage( new Gtk::Table(6, 2) );
  box->pack_start(*table, true, true);
  table->set_spacings(4);
  
  table->attach(*Gtk::manage(new Gtk::Label("Algorithm spec",    Gtk::ALIGN_LEFT)), 0, 1, 0, 1, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Bit multiplier",    Gtk::ALIGN_LEFT)), 0, 1, 1, 2, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Trace count",       Gtk::ALIGN_LEFT)), 0, 1, 2, 3, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Total time",        Gtk::ALIGN_LEFT)), 0, 1, 3, 4, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Scroll speed",      Gtk::ALIGN_LEFT)), 0, 1, 4, 5, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Screen size",       Gtk::ALIGN_LEFT)), 0, 1, 5, 6, Gtk::FILL, Gtk::FILL);
  table->attach(*Gtk::manage(new Gtk::Label("Randomness source", Gtk::ALIGN_LEFT)), 0, 1, 6, 7, Gtk::FILL, Gtk::FILL);
  
  _algorithm_spec.set_tooltip_text("A specification of the line-drawing algorithm to use.  "
				   "Eg, \"1 2 4\" translates to \"10%, 20%, 40%, 20%, 10%\"; "
				   "\"1 1\" translates to \"33.3%, 33.3%, 33.3%\" "
				   "probabilities for each pixel on the next line, centered "
				   "around the previous line's position.");
  _bit_multiplier.set_tooltip_text("Where each descision requires a random bit, use this number of actual bits; must be odd");
  _trace_count.set_tooltip_text("The number of lines to trace down the screen");
  _total_time.set_tooltip_text("The total time of the experiment, in seconds");
  _randomness.set_tooltip_text("Where to get random bits from");
  
  table->attach(_algorithm_spec,   1, 2, 0, 1, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_bit_multiplier,   1, 2, 1, 2, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_trace_count,      1, 2, 2, 3, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_total_time,       1, 2, 3, 4, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_speed,            1, 2, 4, 5, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_screen_size,      1, 2, 5, 6, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  table->attach(_randomness,       1, 2, 6, 7, Gtk::EXPAND|Gtk::FILL, Gtk::FILL);
  
  
  Gtk::VBox *hor_box = Gtk::manage( new Gtk::VBox() );
  box->pack_start(*hor_box, false, true);
  hor_box->pack_start(*Gtk::manage(new Gtk::Label("Horizon")), false, true);
  hor_box->pack_start(_horizon, true, true);
  _horizon.set_tooltip_text("Where to scroll the screen from");
  
  show_all_children();
}

String
Parameters:: get_algorithm_spec() const
{
  return _algorithm_spec.get_algorithm_spec();
}

unsigned
Parameters:: get_bit_multiplier() const
{
  return _bit_multiplier.get_bit_multiplier();
}

unsigned
Parameters:: get_trace_count() const
{
  return _trace_count.get_trace_count();
}

unsigned
Parameters:: get_total_time() const
{
  return _total_time.get_total_time();
}

bool
Parameters:: get_scrolled() const
{
  return _speed.get_scrolled();
}

double
Parameters:: get_speed() const
{
  return _speed.get_speed();
}

double
Parameters:: get_horizon() const
{
  return _horizon.get_horizon();
}

unsigned
Parameters:: get_width() const
{
  return _screen_size.get_width();
}

unsigned
Parameters:: get_height() const
{
  return _screen_size.get_height();
}

String
Parameters:: get_retriever_type() const
{
  return _randomness.get_randomness_source();
}
