/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkdbui/property.h"

using namespace Tk::Db::Ui;

Property:: Property(Tk::Db::Database *database,
		    const Tk::String &domain,
		    const Tk::Db::Id &entity_id,
		    const Tk::String &name,
		    const Tk::Db::Data::Value &value)
  : _database(database),
    _domain(domain),
    _entity_id(entity_id),
    _name(name),
    _value(value),
    _value_widget( Value::create(value) ),
    _apply_button(Gtk::Stock::APPLY)
{
  init();
}

Property:: Property(Tk::Db::Database *database,
		    const Tk::String &domain,
		    const Tk::Db::Id &entity_id,
		    const Tk::String &name,
		    const Tk::Db::Data::Type &type)
  : _database(database),
    _domain(domain),
    _entity_id(entity_id),
    _name(name),
    _value(type),
    _value_widget( Value::create(type) ),
    _apply_button(Gtk::Stock::APPLY)
{
  init();
}

void
Property:: init()
{
  pack_start(*dynamic_cast<Gtk::Widget *>(_value_widget), true, true);
  _value_widget->signal_value_changed().connect( sigc::mem_fun(*this, &Property::value_changed) );
  
  pack_start(_apply_button, false, false);
  _apply_button.set_sensitive(false);
  _apply_button.signal_clicked().connect( sigc::mem_fun(*this, &Property::apply_clicked) );
}

Property:: ~Property()
{
  delete _value_widget;
}

Property::SignalPropertySet
Property:: signal_property_set()
{
  return _signal_property_set;
}

void
Property:: apply_clicked()
{
  _database->set_property(_domain, _entity_id, _name, _value);
  _apply_button.set_sensitive(false);
  _signal_property_set.emit(_domain, _entity_id, _name, _value);
}

void
Property:: value_changed(const Tk::Db::Data::Value &value)
{
  _value = value;
  _apply_button.set_sensitive(true);
}
