/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_QUERY_COMPONENTS_H__
#define __TK_QUERY_COMPONENTS_H__

#include "libtkcommon/tkstring.h"
#include "libtkdb/database.h"

#include "libtkquery/expression.h"

namespace Tk {
  namespace Query {
    
    typedef Tk::String TableName;
    typedef Tk::String TableAlias;
    
    class Query
      {
      public:
      private:
	typedef unsigned TableNo;
	typedef std::map<TableAlias, TableNo> TableAliasNos;
	
	typedef std::pair<TableName, TableAlias> TableDetails;
	typedef std::list<TableDetails> TableList;
	
	static TableAliasNos  _table_alias_nos;
	
	Db::Database         *_database;
	
	TableList             _tables;
	
	Tk::StringList        _link_clauses;
	Tk::Db::Data::Values  _link_params;
	
	Tk::String            _expr;
	Tk::Db::Data::Values  _expr_params;
	
	static TableNo get_table_alias_no(const TableAlias &alias);
	static TableAlias get_table_alias(const TableName &name);
	
      public:
	static TableName get_table_abbrev(const TableName &name);
	
	Query(Db::Database *database);
	void append_tables(const Query &query);
	
	TableAlias add_table(const TableName &name);
	TableAlias add_property_value_table(const Tk::String &domain, const Tk::String &property_type_name);
	void add_link_clause(const Tk::String &clause, const Tk::Db::Data::Values &parameters = Tk::Db::Data::Values());
	void add_link_clause(const Tk::String &alias1, const Tk::String &column1, const Tk::String &alias2, const Tk::String &column2);
	
	void set_expr(const Tk::String &expr, const Tk::Db::Data::Values &parameters = Tk::Db::Data::Values());
	const Tk::String &get_expr() const;
	const Tk::Db::Data::Values &get_expr_params() const;
	
	Tk::String get_sql(const Tk::String &domain) const;
	Tk::Db::PStatement * get_statement(const Tk::String &domain) const;
	
	Tk::Db::Ids get_entities(const Tk::String &domain) const;
      };
    
    typedef std::auto_ptr<Query> QueryPtr;
    
    Tk::Db::Ids get_all_entities(Db::Database *database, const Tk::String &domain);
    
    
  }; /* namespace Query */
}; /* namespace Tk */

#endif /* __TK_QUERY_COMPONENTS_H__ */
