/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <boost/assign/list_of.hpp>

#include "libtkcommon/debug.h"
#include "libtkquery/parse.h"
#include "libtkquery/parse_error.h"

static void
print_expression(Tk::Db::Database *database,
		 const Tk::Query::Expression *expr,
		 const Tk::String &domain,
		 const Tk::String &heading)
{
  Tk::Query::QueryPtr query( expr->build_query(database, domain) );
  Tk::String sql( query->get_sql(domain) );
  
  WDEBUG_STRM << heading
	      << std::endl
	      << sql
	      << std::endl
	      << std::endl;
}

static void
test_string(Tk::Db::Database *database,
	    const Tk::String &domain,
	    const Tk::String &string)
{ 
  try
    {
      Tk::Query::ExpressionPtr expr( Tk::Query::parse_string(domain, string) );
      print_expression(database,
		       expr.get(),
		       domain,
		       Tk::String("=== String query `")
		       + string
		       + Tk::String("' sql in domain `")
		       + domain
		       + Tk::String("' ===")
		       );
    }
  catch (Tk::Query::ParseError &e)
    {
      WDEBUG_STRM << Tk::String("String query `")
		  << string
		  << Tk::String("' does not parse in domain `")
		  << domain
		  << Tk::String("': ")
		  << Tk::String(e.what())
		  << std::endl;
    }
}

int main(int argc, char **argv)
{
  Tk::String data_source_name("TkResolver++");
  Tk::String username("psi");
  Tk::String password("isp");

  Tk::Db::Database database(data_source_name, username, password);
  
  for (int i = 1; i < argc; ++i)
    {
      Tk::Query::ExpressionPtr expr( Tk::Query::parse_file("experiment", argv[i]) );
      
      Tk::String heading("=== Query file ");
      heading += Tk::String(argv[i]);
      heading += Tk::String(L" sql ===");
      
      print_expression(&database, expr.get(), "experiment", heading);
    }

  Tk::String query("[algorithm spec] = \"1 1\"");
  for (unsigned i(1); i <= query.size(); ++i)
    {
      Tk::String test_str( query.substr(0, i) );
      test_string(&database, "experiment", test_str);
    }
  
  test_string(&database, "location", "name = \"44 Greville Road\"");
  test_string(&database, "operator", "oper.[send results] = true");
  test_string(&database, "operator", "oper.[send results] = true & \nl.[name] = \"blah\" ");
  test_string(&database, "experiment", "o.name like \"%Joe%\"");
}
