/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_TKRESOLVER_H__
#define __TK_TKRESOLVER_H__

#include <list>
#include <utility>

#include "libtkcommon/tkstring.h"
#include "libtkcommon/tkvalue.h"
#include "libtkalgorithm/algorithm.h"
#include "libtkrandom/decision_tree.h"
#include "libtkrandom/randomness.h"
#include "sdl.h"
#include "display.h"
#include "types.h"

namespace Tk {

  class TKResolver
  {
  public:
    typedef Uint32 Time; // milliseconds

    static const short MILLISECONDS_PER_SECOND = 1000;

  private:
    unsigned             _trace_count;
    unsigned             _total_time; /* seconds */

    SDL                  _sdl;
    Display            * _display;
    Algorithm            _algorithm;
    Random::Randomness   _rand;
    Random::DecisionTree _decision_tree;
    bool                 _test;

    Line                 _current_line;
    std::list<Line>      _offsets;
    Time                 _actual_time;
  
    /** returns true on quit command */
    bool control();
    void slice();
    
  public:
    TKResolver(const String &retriever_type,
	       const String &algo,
	       unsigned bit_multiplier,
	       unsigned trace_count,
	       unsigned total_time,
	       unsigned width,
	       unsigned height,
	       bool scrolled = false,
	       double speed = 0.0,
	       double horizon = 0.0,
	       bool test = true);
    ~TKResolver();

    unsigned get_display_width() const;
    unsigned get_display_height() const;
    unsigned get_total_lines() const;
    const Random::Bits & get_used_bits() const;
    void run();
    const Result & get_result() const;
    const Time & get_actual_time() const;
  };

}; /* namespace Tk */

#endif /* __TK_TKRESOLVER_H__ */
